#
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#
# ======================================================================
# mrbump.tcl --
#
# CCP4Interface 
#
# =======================================================================

############################################################################
proc mrbump_setup { typedefVar arrayname } {
############################################################################
  upvar #0  $typedefVar typedef
  upvar #0 $arrayname array

  DefineMenu _mrbump_mode [ list "Model search and Molecular Replacement" \
						  "Model search only" ] \
	     [ list FULLMR MODELS ] 

  DefineMenu  _ma_program [ list "Mafft"  "Clustalw" ] \
              [ list  MAFFT CLUSTALW ]

  set typedef(_tryall)  { menu  { "first solution is produced using a search model" \
	                          "all of the search models have been tried in MR" } \
					{ 0 1 } }

  return 1
}


############################################################################
proc mrbump_run { arrayname } {
############################################################################
  upvar #0 $arrayname array
 
# Change OUTPUT_FILES if we are running in models only mode
   if { $array(MRBUMP_MODE) == "Model search only" } {
       set array(OUTPUT_FILES) ""
   } elseif { $array(MRBUMP_MODE) == "Model search and Molecular Replacement" } {
       set array(OUTPUT_FILES) "HKLOUT XYZOUT"
       if { $array(USE_MOLREP) == 1 } { 
          if { $array(USE_PHASER) == 0 } { 
             set array(MR_PROGRAM_LIST) "MOLREP"
          } else {
             set array(MR_PROGRAM_LIST) "MOLREP PHASER"
          }
       } else { 
          if { $array(USE_PHASER) == 0 } { 
             set array(MR_PROGRAM_LIST) "MOLREP"
          } else {
             set array(MR_PROGRAM_LIST) "PHASER"
          }
       }
   }

# Sanity Checks:

  # Make sure number of models to be used in MR is <= number that have been prepared
  if { $array(MRBUMP_MODE) == "Model search and Molecular Replacement" } {
     if { $array(USE_PHASER) == 1 } { 
        if { $array(ENSEMNUM) > $array(MRNUM) } {
               WarningMessage "Input Error: Number of models used in Phaser Ensemble should be <= Number of models used in MR."
               return 0
        }
     }
  }

  # Check for multiple alignment program
  if { [FindExecutable [ string tolower $array(MAPROGRAM) ] ] == "" } {
         # Not found
         WarningMessage "Input Error: $array(MAPROGRAM) program not found in system path."
         return 0
     }

  if { $array(MRBUMP_MODE) == "Model search and Molecular Replacement" } {
     # Check for Molrep program
     if { $array(USE_MOLREP) == 1 } { 
        if { [FindExecutable [ string tolower "molrep" ] ] == "" } {
            # Not found
            WarningMessage "Input Error: Molrep program not found in system path."
            return 0
        }
     }
   
     # Check for Phaser program
     if { $array(USE_PHASER) == 1 } { 
        if { [FindExecutable [ string tolower "phaser" ] ] == "" } {
            # Not found
            WarningMessage "Input Error: Phaser program not found in system path."
            return 0
        }
     }
  }

  # Check for perl 
  if { [FindExecutable [ string tolower "perl" ] ] == "" && $array(SSMSEARCH) == 1 } {
         # Not found
         WarningMessage "Input Error: Perl not found in system path. Perl is required for the SSM search. Unselect it and retry."
         return 0
     }

  # Setup the IGNORE keyword. These are the PDB ids to ignore in the search
  set array(IGNORE) ""
  for {set N 1} {$N <= $array(NPDBIDS)} {incr N 1} {
    lappend array(IGNORE) $array(PDBID,$N)
  }
 
  # Setup the INCLUDE keyword. These are the PDB ids to include in the search
  set array(INCLUDE) ""
  for {set N 1} {$N <= $array(NCHAINIDS)} {incr N 1} {
    lappend array(INCLUDE) $array(CHAINID,$N)
  }
 
  # Set the root directory
  set array(ROOTDIR) [GetDefaultDirPath]
  
  return 1
}


############################################################################
proc pdbCodes { arrayname counter } {
############################################################################
  upvar #0 $arrayname array

  SetProgramHelpFile "mrbump" 
  
  CreateLine line \
    help match \
    message "Define PDB codes to ignore" \
    label "PDB id" \
    widget PDBID 

}

############################################################################
proc chainCodes { arrayname counter } {
############################################################################
  upvar #0 $arrayname array

  SetProgramHelpFile "mrbump" 
  
  CreateLine line \
    help match \
    message "Define Chain id codes to include" \
    label "Chain id" \
    widget CHAINID 

}

############################################################################
proc mrbump_task_window { arrayname } {
############################################################################
  upvar #0 $arrayname array

  if { [CreateTaskWindow $arrayname \
       "MrBUMP: Automated Model generation and Molecular Replacement" "MrBUMP" \
       [ list "Template Search Options" \
              "Search Model Preparation Options" \
              "Molecular Replacement and Refinement Options" \
              "Development Options" \
       ] ] == 0 } return

  SetProgramHelpFile "mrbump"

#=PROTOCOL==============================================================

  OpenFolder protocol 

  CreateTitleLine line TITLE

  CreateLine line \
      message "Select the mode for MrBUMP to run" \
      label "Program Mode:" \
      widget MRBUMP_MODE 

#=FILES================================================================ 

  OpenFolder file

  # Input MTZ file
  CreateInputFileLine line \
     "Enter name of the target MTZ file" \
     "MTZ in" \
     HKLIN DIR_HKLIN \
      -fileout HKLOUT DIR_HKLOUT "_mrbump_soln" \
      -fileout XYZOUT DIR_XYZOUT "_mrbump_soln" \
      -setfileparam space_group_name SPACE_GROUP

  CreateLabinLine line \
	"Observed amplitude (FP) and obligatory sigma (SIGFP)" \
	HKLIN "F    " F  [list F F.F_sigF.F] \
        -sigma "Sigma  " SIGF  [list F F.F_sigF.sigF ]

  CreateLabinLine line \
      "Choose Free-R flag" \
      HKLIN "Free-R" FreeR_flag  [list FreeR_flag]

  OpenSubFrame frame \
      -toggle_display MRBUMP_MODE open [list MODELS]

  CreateLine line \
      message "Spacegroup read from the input MTZ file" \
      label "Spacegroup from MTZ file:" \
      varlabel SPACE_GROUP 

  CloseSubFrame

  OpenSubFrame frame \
      -toggle_display MRBUMP_MODE open [list FULLMR]

  CreateLine line \
      message "Spacegroup read from the input MTZ file" \
      label "Spacegroup from MTZ file:" \
      varlabel SPACE_GROUP \
      label "Note that MrBUMP will assume this spacegroup for the MR stage" -italic

  CloseSubFrame

  # Input Sequence file
  CreateInputFileLine line \
      "Enter name of the target sequence file" \
      "SEQ in" \
      SEQIN DIR_SEQIN

  OpenSubFrame frame \
      -toggle_display MRBUMP_MODE open [list FULLMR]

  CreateOutputFileLine line \
      "Output MTZ File for solution" \
      "MTZ out" \
      HKLOUT DIR_HKLOUT

  CreateOutputFileLine line \
      "Output PDB File for solution" \
      "PDB out" \
      XYZOUT DIR_XYZOUT

  CloseSubFrame

  CreateLine line \
      message "Number of molecules in the asu" \
      help nmasu \
      label "Number of molecules in the asu:" \
      widget NMASU -width 3 \
      label "Leave blank for automatic calculation" -italic

#=Template Search Options========================================================== 

  OpenFolder 1 open 

  CreateLine line \
      message "Multiple alignment program" \
      label "Multiple alignment program:" \
      widget MAPROGRAM 

  CreateLine line \
      message "E-value for Fasta search" \
      help evalue \
      label "E-value for Fasta search" \
      widget EVALUE -width 7 

  CreateLine line \
      message "Database Updating" \
      help update \
      widget UPDATE -width 4 \
      label "Update local copies of search databases" 

  CreateLine line \
      message "Run fasta sequence-based search locally" \
      help fasta_local \
      widget FASTALOCAL -width 4 \
      label "Run the fasta search locally." \
      label "Requires fasta34 to be installed" -italic

  CreateLine line \
      message "Use SCOP (domain-based search) to search for templates" \
      help use_scop \
      label "Additional search methods to use:" \
      widget SCOPSEARCH -width 4 \
      label "SCOP    " \
      message "Use PQS (multimeric search) to search for templates" \
      help use_pqs \
      widget PQSSEARCH -width 4 \
      label "PQS    " \
      message "Use SSM (secondary structure-based search) to search for templates" \
      help use_ssm \
      widget SSMSEARCH -width 4 \
      label "SSM    " 

  CreateLine line \
      label "Enter Chain id codes to be included in the template model search: (e.g. 1nio_A)"

  CreateExtendingFrame NCHAINIDS chainCodes \
      "Add Chain id codes to include in template search" \
      "Add Chain id" \
      CHAINID  

#=Model Preperation Options================================================================ 

  OpenFolder 2 open

  CreateLine line \
      message "Number of search matches from which to generate search models" \
      help num_mr \
      label "Maximum number of search results from which to generate search models for use in MR:" \
      widget MRNUM -width 3 

  CreateLine line \
      message "Generate search models using the PDBClip method" \
      label "Search models to create:" \
      help mdld \
      widget MDLD -width 4 \
      label "PDBClip    " \
      message "Generate search models using the Molrep method" \
      help mdlm \
      widget MDLM -width 4 \
      label "Molrep    " \
      message "Generate search models using the Chainsaw method" \
      help mdlc \
      widget MDLC -width 4 \
      label "Chainsaw    " \
      message "Generate search models using the Polyalanine method" \
      help mdlp \
      widget MDLP -width 4 \
      label "Polyalanine    " 

#  CreateLine line \
      message "Generate search models using the Molrep method" \
      help mdlm \
      widget MDLM -width 4 \
      label "Create Molrep search models" 

#  CreateLine line \
      message "Generate search models using the Chainsaw method" \
      help mdlc \
      widget MDLC -width 4 \
      label "Create Chainsaw search models" 

#  CreateLine line \
      message "Generate search models using the Polyalanine method" \
      help mdlp \
      widget MDLP -width 4 \
      label "Create Polyalanine search models" 

#=Molecular Replacement and Refinement Options======================================================== 

  OpenFolder 3 \
      MRBUMP_MODE open [list FULLMR] hide

  CreateLine line \
      message "Try search models in Molrep" \
      help use_molrep \
      label "Molecular replacement programs to try:" \
      widget USE_MOLREP -width 4 \
      label "Molrep    " \
      message "Try search models in Phaser" \
      help use_phaser \
      widget USE_PHASER -width 4 \
      label "Phaser    " 

  OpenSubFrame frame \
      -toggle_display USE_PHASER open 1

  CreateLine line \
      message "Number of prepared models to use in Phaser ensemble (ENSEMNUM)" \
      help num_ensem \
      label "Maximum number of prepared models to use in Phaser Ensemble:" \
      widget ENSEMNUM -width 3 

  CreateLine line \
      message "Number of clashes tolerated by Phaser (PACK keyword)" \
      help pack \
      label "Number of clashes to tolerate in Phaser:" \
      widget PACK -width 3 

  CloseSubFrame

  CreateLine line \
      message "Number of cycles of restrained refinement in Refmac (NCYC keyword)" \
      help ncyc \
      label "Number of cycles of restrained refinement in Refmac:" \
      widget NCYC -width 3 

#  CreateLine line \
      message "If set to true the program will try all of the search models in molecular replacement." \
      help tryall \
      widget TRYALL -width 4 \
      label "Try all search models (don't exit when the first solution is found)." 

  CreateLine line \
      help  tryall \
      message "Choose when you want to exit the MR stage." \
      label "Finish when" \
      widget TRYALL 

#=Development Options======================================================== 

  OpenFolder 4 closed 

  CreateLine line \
      label "Enter PDB id codes to be ignored in the template model search: (e.g 1nio)"

  CreateExtendingFrame NPDBIDS pdbCodes \
      "Add PDB id codes to ignore in template search" \
      "Add PDB id" \
      PDBID  

  CreateLine line \
      message "Enable HTML output of the results as they are generated." \
      help htmlout \
      widget HTMLOUT -width 4 \
      label "HTML file output of results" 

  CreateLine line \
      message "Debug mode (gives more verbose output)" \
      help debug \
      widget DEBUG -width 4 \
      label "Use debug mode." \
      label " Gives a more verbose output" -italic

  CreateLine line \
      message "Isolated mode (run without web connection - development only)" \
      help isolated \
      widget ISOLATED -width 4 \
      label "Isolated mode." \
      label " Development use only" -italic

}
