#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#
#
# Program to calculate the number of molecules in the a.s.u. based on the 
# cell dimensions, resolution, and space group of the target structure. Derived
# from the method outlined by Kantardjieff and Rupp (2000).
#
# Ronan Keegan 21/6/05
# 

import os

if not os.environ.has_key('CCP4'):
    raise RuntimeError, 'CCP4 not found'

import sys
import string
import time 
import popen2
import Fasta_parse
import MTZ_parse

class Nmol_calc(Fasta_parse.Fasta_parse, MTZ_parse.MTZ_parse):
   ''' A class to calculate the number of molecules in the a.s.u.'''

   def __init__(self):
      MTZ_parse.MTZ_parse.__init__(self)
      Fasta_parse.Fasta_parse.__init__(self)

      self.nmol=1
      self.score=0.0
      self.Vm_init=0.0
      self.nmol_max=10

      self.matt_coef_list=dict([])
      self.solvent_list=dict([])
      self.deviation=dict([])
      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False
  
   def setDEBUG(self, flag):
      self.debug=flag 

   def set_VM_init(self, resolution):
      ''' A function to work out the most likely value for Vm given the resolution. '''

      # Set-up a best guess for the value of Vm based on the resolution of the data.  

      if resolution < 0.54:
         print "Resolution too high. Try again."
         sys.exit(1)
      elif resolution >= 0.54 and resolution < 1.21:
         self.Vm_init=2.15
      elif resolution >= 1.21 and resolution < 1.51:
         self.Vm_init=2.3
      elif resolution >= 1.51 and resolution < 1.81:
         self.Vm_init=2.45
      elif resolution >= 1.81 and resolution < 2.01:
         self.Vm_init=2.6
      elif resolution >= 2.01 and resolution < 2.41:
         self.Vm_init=2.7
      elif resolution >= 2.41 and resolution < 2.81:
         self.Vm_init=2.95
      else:
         print "Resolution too low for this method."
         sys.exit(1)

   def calculate_nmol(self, MTZIN, SEQIN):
      ''' A function to calculate the number of molecules in the a.s.u. Take in 
          resolution, cell parameters, symmetry group and molecular weight as 
          arguements. Calls matthews_coef. ''' 

      # Parse sequence file and mtz file.

      self.parseSeqFile(SEQIN)
      self.calc_MW()
      self.run_mtzdmp(MTZIN)

      # Work out the initial estimate for Vm.

      # self.set_VM_init(self.resolution)
       
      # Start a sub-process to run matthes_coef. We do this several times to test for the best possible
      # answer. Change nmol_max to vary the number of tests.

      matt_c_OUT,matt_c_IN=popen2.popen2("matthews_coef")
      matt_c_IN.write('CELL %.3f %.3f %.3f %.3f %.3f %.3f\n' % (self.cell_dimensions['a'], self.cell_dimensions['b'], \
         self.cell_dimensions['c'], self.cell_dimensions['alpha'], self.cell_dimensions['beta'], self.cell_dimensions['gamma'],))  
      matt_c_IN.write('SYMM %s\n' % self.space_group)  
      matt_c_IN.write('MOLW %.3f\n' % self.mol_weight)  
      matt_c_IN.write('RESO %.3f\n' % self.resolution)  
      matt_c_IN.write('AUTO\n')  
      #matt_c_IN.write('XMLOUTPUT\n')  
      matt_c_IN.write('END\n')  
      matt_c_IN.close() 

      top_score = 0.0
      line = matt_c_OUT.readline()
      while line:
         if "Matthews Coeff  %solvent" in line:
            line = matt_c_OUT.readline()
            line = matt_c_OUT.readline()
            while line:

               if "__________________" in line:
                  break

               results = string.split(line)
               if float(results[4]) > top_score:
                  top_score = float(results[4])
                  nmol_estimate = int(results[0])
                  Vm_estimate = float(results[1])
                  solvent_estimate = float(results[2])
                  score_reso = float(results[3])
                  score_tot = float(results[4])

               line = matt_c_OUT.readline()

         line = matt_c_OUT.readline()
        
      matt_c_OUT.close()

      #os.wait()
      time.sleep(3)

      print "Matthews_Coef message: Estimated number of molecules: %i" % nmol_estimate 
      print "Matthews_Coef message: Score for this number of molecules: %.3f" % score_tot
      print "Matthews_Coef message: Actual Vm: %.3f and Solvent content: %.3f %%" % (Vm_estimate, solvent_estimate)

      return nmol_estimate  


if __name__ == '__main__':

      if len(sys.argv) != 3:
         print "Usage: <mtz file> <sequence file (fasta format)>"
         sys.exit(1)     
      else:
         MTZIN = sys.argv[1]
         SEQIN = sys.argv[2]
 
      nmol = Nmol_calc()
      NMOL = nmol.calculate_nmol(MTZIN, SEQIN)
         
