#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan 
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#
# Initialisation for a MRBUMP job
# Ronan Keegan - 23.10.05

import os, sys, string
import shutil
import MRBUMP_keywords
import MRBUMP_environment
import MRBUMP_update_DB
import MRBUMP_check_dependencies

class Initialise:
   """ A class to do the initialisation for a MRBUMP job. This includes:
       1) Read the command line arguments,
       2) Read in the keywords from command prompt or a file,
       3) Read the environment variables,
       4) Setup the search directory structure,
       5) Update any search databases. """
   
   def __init__(self):
      self.keywords=MRBUMP_keywords.Keywords()
      self.environment=MRBUMP_environment.Environment()
      self.check_deps=MRBUMP_check_dependencies.Check_deps()
      self.db_tools=MRBUMP_update_DB.DB_tools()

      self.version="0.3.1"
      self.version_date="Sat Jul  8 14:52:40 2006"
      self.hklin = ""
      self.seqin = ""
      self.hklout = ""
      self.xyzout = ""

      self.hklin_filename = ""
      self.seqin_filename = ""
      self.hklout_filename = ""
      self.xyzout_filename = ""
 
      self.labin_found = False
      self.jobid_found = False

      self.mrbump = ""
      self.mrbump_incl = ""
      self.mrbump_data = ""
 
      self.current_dir = ""
      self.search_dir = ""
      self.results_dir = ""
      self.scratch_dir = ""

      self.model_types=[]

      # Results html has a variable location to support inclusion in the CCP4i gui.

      self.results_html = ""
      self.results_html_filename = ""

      # Relative link is a variable to hold the path to the results html files.
      # It should be empty for operation when the results html is in its 
      # default location. This is to ensure that the WebService works correctly.

      self.relative_link = ""

   def setHKLIN(self, filename):
      self.hklin = filename
      self.hklin_filename = string.split(filename,'/')[-1]

   def setSEQIN(self, filename):
      self.seqin = filename
      self.seqin_filename = string.split(filename,'/')[-1]

   def setHKLOUT(self, filename):
      self.hklout = filename
      self.hklout_filename = string.split(filename,'/')[-1]

   def setXYZOUT(self, filename):
      self.xyzout = filename
      self.xyzout_filename = string.split(filename,'/')[-1]

   def setMRBUMP(self, directory):
      self.mrbump = directory
   
   def setMRBUMP_INCL(self, directory):
      self.mrbump_incl = directory
   
   def setMRBUMP_DATA(self, directory):
      self.mrbump_data = directory

   def setSearch_DIR(self, directory):
      self.search_dir = directory

   def setResults_DIR(self, directory):
      self.results_dir = directory

   def setScratch_DIR(self, directory):
      self.scratch_dir = directory

   def setResults_HTML(self, filename):
      self.results_html = filename

   def setResults_HTML_Filename(self, filename):
      self.results_html_filename = filename

   def setRelative_Link(self, directory):
      self.relative_link = directory

   def mrbump_initialisation(self, arguments):
      """ A function to initialise a MrBUMP job. """ 
     
      print ""
      print "#########################################################################"
      print "#########################################################################"
      print "#########################################################################"
      print "#  CCP4: MRBUMP - Automated Bulk Molecular Replacement (version 0.3.1)  #"
      print "#########################################################################"
      print ""
      print "  Version date: Sat Jul  8 14:52:40 2006"
      print ""

      # Read the command line arguments
      self.read_cmdline(arguments)
    
      # Read the keywords from the STDIN or from a file
      self.keywords.read_keywords()

      # Check to see if a new version of mrbump exists
      if self.keywords.UPDATE:
         self.db_tools.check_version(self.version)

      # Output the mode of operation details
      sys.stdout.write("\n")
      sys.stdout.write("************************************************************************************************\n")
      if self.keywords.ONLYMODELS == False:
         sys.stdout.write("Mode of operation: FULLMR - Search models will be created and used in Molecaular Replacement.\n")
      else:
         sys.stdout.write("Mode of operation: MODELS - Search models will be created only.\n")
      sys.stdout.write("************************************************************************************************\n")
      sys.stdout.write("\n")

      # Output the input and output file details
      sys.stdout.write("Input MTZ file: %s\n" % self.hklin)
      sys.stdout.write("Input Sequence file: %s\n" % self.seqin)

      if self.keywords.ONLYMODELS == False:
         sys.stdout.write("Output MTZ file: %s\n" % self.hklout)
         sys.stdout.write("Output PDB file: %s\n" % self.xyzout)
      sys.stdout.write("\n")

      # If we are running in ISOLATED mode we need to set some other variables
      if self.keywords.ISOLATED:
       
         print "Running in ISOLATED mode. Turning off SSM, SCOP and PQS searches."
         print "Setting UPDATE to False and FASTALOCAL to True"
         print ""

         self.keywords.SSM = False
         self.keywords.PQS = False
         self.keywords.UPDATE = False
         self.keywords.fasta_local = True
         
         # Test to see if a /tmp/mrbump_pdb_files folder has been created

         if os.path.isdir("/tmp/mrbump_pdb_files") == False:
            print "Error: to run in ISOLATED mode you need to create a folder: /tmp/mrbump_pdb_files"
            print "This should contain all of the PDBs that will be used in the job."
            print ""
            sys.exit()
     
      # Check that at least one Model preperation technique has been specified. If none are set PDBCLP 
      # to True.
      if self.keywords.MDLDPDBCLP == False and self.keywords.MDLPLYALA == False \
         and self.keywords.MDLMOLREP == False and self.keywords.MDLCHAINSAW == False:
            print "No model preparation method has been specified. PDBClip will be used in this case."
            self.keywords.MDLDPDBCLP = True

      # Add the types of models we want to generate to the list of model types
      if self.keywords.MDLDPDBCLP:
         self.model_types.append('PDBCLP')
      if self.keywords.MDLPLYALA:
         self.model_types.append('PLYALA')
      if self.keywords.MDLMOLREP:
         self.model_types.append('MOLREP')
      if self.keywords.MDLCHAINSAW:
         self.model_types.append('CHNSAW')
 
      # Setup the environment for python
      self.environment.read_environment()

      # Check that all dependencies are in place

      # Sequence alignment program:
      self.check_deps.chk_ma_program(self.keywords)
      # Molecular Replacement program:
      self.keywords.MR_PROGRAM_LIST=self.check_deps.chk_mr_program(self.keywords)
      
      # Setup the directory structure for the job
      self.make_job_dir()

      # Copy in the input files to the search directory - .seq, .mtz.
      # After copying, reassign the file pointers.

      shutil.copy(self.seqin, self.search_dir + '/input/')

      # Fix for permissions problem with sequence file. If the permissions on your sequence file are
      # read only then it can't be re-written for molrep. 
      os.chmod(self.search_dir + '/input/' + self.seqin_filename, 0755) 

      shutil.copy(self.hklin, self.search_dir + '/input/')

      self.seqin=self.search_dir + '/input/' + self.seqin_filename
      self.hklin=self.search_dir + '/input/' + self.hklin_filename

      # Copy over header for results page
      shutil.copyfile(self.mrbump_incl + '/web_template/ccp4.gif', self.results_dir + '/images/ccp4.gif')
      shutil.copyfile(self.mrbump_incl + '/web_template/cclrc-logo.jpg', self.results_dir + '/images/cclrc-logo.jpg')
      shutil.copyfile(self.mrbump_incl + '/web_template/slide_bar.gif', self.results_dir + '/images/slide_bar.gif')
      shutil.copyfile(self.mrbump_incl + '/web_template/results.html', self.results_html)

      # Update the search databases if necessary
      self.db_tools.setMRBUMPDataDir(self.mrbump_data)

      if self.keywords.run_local == False:
         self.db_tools.test_first_run(self.environment)
         self.db_tools.test_databases(self.environment, self.keywords.UPDATE)
      else:
         self.db_tools.test_first_run(self.environment)
         self.keywords.SSM = False
         self.keywords.PQS = False

   def read_cmdline(self, arguments):
      """ A function to read in the command line arguments. """
      
      # Check the number of arguments is correct
      if len(arguments) != 9 and len(arguments) != 5: 
         self.usage() 
         sys.exit(1)

      # If only HKLIN and SEQIN are given set the MODE to MODELS ONLY
      if len(arguments) == 5: 
         self.keywords.ONLYMODELS = True

      # Test to see if the HKLIN variable is specified 
      #if sys.argv[1].upper() != 'HKLIN':
      if arguments[1].upper() != 'HKLIN':
         self.usage() 
         sys.exit(1)

      # Test to see if the SEQIN variable is specified 
      #if sys.argv[3].upper() != 'SEQIN':
      if arguments[3].upper() != 'SEQIN':
         self.usage() 
         sys.exit(1)

      self.setHKLIN(arguments[2])
      self.setSEQIN(arguments[4])

      if self.keywords.ONLYMODELS:
         self.setHKLOUT("null.mtz")
         self.setXYZOUT("null.pdb")
      else:
         # Test to see if the HKLOUT variable is specified 
         if arguments[5].upper() != 'HKLOUT':
            self.usage() 
            sys.exit(1)

         # Test to see if the XYZOUT variable is specified 
         if arguments[7].upper() != 'XYZOUT':
            self.usage() 
            sys.exit(1)

         self.setHKLOUT(arguments[6])
         self.setXYZOUT(arguments[8])

   def make_job_dir(self):
      """ A function to create the directory three for the MRBUMP job. """

      # Get the current working directory
      self.current_dir=os.getcwd()

      # Set the master directory
      self.setSearch_DIR(self.keywords.ROOTDIR + '/search_' + self.keywords.job_ID)

      # Set the results directory
      self.setResults_DIR(self.search_dir + '/results')

      # Also set the results.html file location
      if self.keywords.results_html == "":
         self.setResults_HTML(self.results_dir + '/results.html')
         self.setResults_HTML_Filename('results.html')
         self.setRelative_Link("")
      else:
         self.setResults_HTML(self.keywords.results_html)
         if os.name == 'nt':
            # Set results html filename under windows
            self.setResults_HTML_Filename((string.split(self.keywords.results_html,'\\'))[-1])
         else:
            # Set results html filename under non-windows OS
            self.setResults_HTML_Filename((string.split(self.keywords.results_html,'/'))[-1])
         self.setRelative_Link(self.results_dir + '/')

      # Set the scratch directory
      self.setScratch_DIR(self.search_dir + '/scratch')

      # Setup the directory structure
      if os.path.isdir(self.search_dir):
         print ""
         print "A directory for a job with this name already exists:\n " \
            + self.search_dir
         print "Please delete it or choose a different job name"
         print ""
         sys.exit(1)
      else:
         os.mkdir(self.search_dir)
         os.mkdir(self.search_dir + '/input')
         os.mkdir(self.search_dir + '/logs')
         os.mkdir(self.search_dir + '/scratch')
         os.mkdir(self.search_dir + '/data')
         os.mkdir(self.search_dir + '/sequences')

      # Create the results directory and all of its subfolders
      if not os.path.isdir(self.results_dir):
         os.mkdir(self.results_dir)
      os.mkdir(self.results_dir + '/images')
      os.mkdir(self.results_dir + '/data')
      os.mkdir(self.results_dir + '/data/sequences')
      os.mkdir(self.results_dir + '/pdb_files')
      os.mkdir(self.results_dir + '/log_files')
      os.mkdir(self.results_dir + '/marginal_solns')

   def usage(self):
      sys.stdout.write("\n")
      sys.stdout.write("Usage:\n")
      sys.stdout.write("Full Model generation and MR:\n  mrbump HKLIN foo_in.mtz SEQIN foo.seq HKLOUT foo_out.mtz XYZOUT foo.pdb\n")
      sys.stdout.write("Model generation only:\n  mrbump HKLIN foo_in.mtz SEQIN foo.seq\n")
      sys.stdout.write("\n")

if __name__ == '__main__':
 
   init=Initialise()
   init.mrbump_initialisation()
   print init.keywords.E_value
 
