# /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan 
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#
#A script to update the various search databases
#1. The PDB fasta sequence database file,
#2. The 3 PQS multimer listing files,
#3. The SCOP domain listing file.
#Ronan Keegan 27.9.05

import os, sys, string
import urllib
import shutil

class DB_tools:
   """ A class to carry out various functions relating to the search databases. """
   
   def __init__(self):
      self.mrbump_data_dir = ""
      self.db_file_list=['pdb_ATOMseqs.txt', 'pqs_LIST.txt', 'pqs_ASALIST.txt', 'pqs_BIOLIST.txt', 'scop_list.txt']
      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False

   def setDEBUG(self, flag):
      self.debug = flag  

   def setMRBUMPDataDir(self, directory):
      self.mrbump_data_dir = directory

   def test_first_run(self, environment):
      """ A function to setup directories if this is the first run ever of MRBUMP. """

      # Test for the "~/.ccp4_mrbump" folder.
   
      if os.path.isdir(environment.SETUP_DIR) == False:
         print "\n"
         print "Welcome to MRBUMP"
         print "This appears to be your first time running MRBUMP. Creating a '.ccp4_mrbump' folder in your home folder"
         print "This will hold copies of the various database files required for the template searches" 
         print "\n"

         os.mkdir(environment.SETUP_DIR) 
         for file in self.db_file_list:
            shutil.copy(self.mrbump_data_dir + '/' + file, environment.SETUP_DIR + '/' + file)
      
      # If .ccp4_mrbump does exist, make sure it contains all of the DB files.
      else:
         for file in self.db_file_list:
            if os.path.isfile(environment.SETUP_DIR + '/' + file) == False:
               shutil.copy(self.mrbump_data_dir + '/' + file, environment.SETUP_DIR + '/' + file)
    
   def test_databases(self, environment, update):
      """ A function to update all of the databases used in the template search. """

      # Test the database files to see if they need updating.
 
      if update == True: 
         print "\nDatabase update: Checking the web to see if newer versions of the search databases exist."

         self.update_DB('Fasta DB', 'http://www.ccp4.ac.uk/ronan/bmp_data/pdb_ATOMseqs.txt', environment.SETUP_DIR + '/pdb_ATOMseqs.txt')
         self.update_DB('PQS List DB', 'http://www.ccp4.ac.uk/ronan/bmp_data/pqs_LIST.txt', environment.SETUP_DIR + '/pqs_LIST.txt')
         self.update_DB('PQS ASAList DB', 'http://www.ccp4.ac.uk/ronan/bmp_data/pqs_ASALIST.txt', environment.SETUP_DIR + '/pqs_ASALIST.txt')
         self.update_DB('PQS BIOList DB', 'http://www.ccp4.ac.uk/ronan/bmp_data/pqs_BIOLIST.txt', environment.SETUP_DIR + '/pqs_BIOLIST.txt')
         self.update_DB('SCOP List DB', 'http://www.ccp4.ac.uk/ronan/bmp_data/scop_list.txt', environment.SETUP_DIR + '/scop_list.txt')
      else:
         print '\nDatabase update: Keyword "UPDATE" set to False. MrBUMP will not attempt a database update.'
 

   def update_DB(self, DB_type, url, local_file):
      """ A function to update the search databases if they are out of date."""
   
      if self.debug:
         print "\nDatabase update: Checking if " + DB_type + " database needs updating..."

      # Get the size of the local file and the remote file, if they differ retrieve the new file
      u=urllib.urlopen(url)
      url_size=int(u.info()['content-length'])
      u.close()

      # If on Windows change the "C:" in the file name to "C|"
      if os.name == "nt":
         local_file="file:///" + local_file.replace(":","|")
      else:
         local_file="file://" + local_file

      l=urllib.urlopen(local_file)
      local_size=int(l.info()['content-length'])
      l.close()
   
      if self.debug:
         print "Database update: remote URL: %s" % url
         print "Database update: remote URL size: " + `url_size`
         print "Database update: local file size: " + `local_size`

      # Download the latest file
      if url_size > local_size:
         print "\nDatabase update: Your " + DB_type + " file is out of date, attempting an update..."
         try:
            urllib.urlretrieve(url ,local_file)
            urllib.urlcleanup()
         except:
            print "Download error: Cannot retrieve new version of " + DB_type + ". Will use old version"
      else:
         print "Database update: " + DB_type + " is up to date."

   def check_version(self, version):
      """ A function to check to see if a new version of MrBUMP exits."""

      download_url="http://www.ccp4.ac.uk/MrBUMP/release"

      # Retrieve the latest version number
      try:
         v=urllib.urlopen(download_url + "/version.txt")
         new_version_string=string.strip(v.readline())
         v.close()
      except:
         sys.stdout.write("New version check failed\n")
         sys.stdout.write("\n")
         return 

      # Set it as a float for comparison with current version 
      if new_version_string=="":
         new_version_string="0.0"

      head=new_version_string[0:3]
      tail=new_version_string[3:]

      try:
         new_version=float(head + tail.replace(".",""))
      except:
         sys.stdout.write("New version check failed\n")
         sys.stdout.write("\n")
         return 

      # Set the current version
      if version=="":
         version="0.0"

      head=version[0:3]
      tail=version[3:]

      cur_version=float(head + tail.replace(".",""))

      if new_version > cur_version:
         sys.stdout.write("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n")
         sys.stdout.write("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n")
         sys.stdout.write("   A newer version of MrBUMP is available (version " + new_version_string + ")\n")
         sys.stdout.write("   This can be downloaded from:\n")
         sys.stdout.write("   " + download_url + "/mrbump-" + new_version_string + ".tar.gz\n")
         sys.stdout.write("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n")
         sys.stdout.write("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n")
         sys.stdout.write("\n")


