#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
# Top level wrapper for launcing Molecular Replacement and Refinement jobs
# Ronan Keegan 10/03/06
#

import os, sys, string
import Model_Queue
import MRBUMP_Phaser
import MRBUMP_Molrep
import Ensemble


class MR_submit:

   def __init__(self):
      self.name=""
      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False

   def setDEBUG(self, flag):
      self.debug=flag

   def MR_start(self, target_info, init, mstat):
      """ A function to submit a molrep job to an SGE queue. """

      sys.stdout.write("--------------------------------------------------------\n")
      sys.stdout.write("Molecular Replacement log message: Launching MR jobs\n")
      sys.stdout.write("--------------------------------------------------------\n")

      # Sort the model list so that jobs with a higher sequence identity are started first

      self.sort_model_list(mstat)
      job_id_log = open(mstat.search_dir + '/logs/jobs.txt','w')
 
      # Setup the Ensemble model for Phaser
      if mstat.MRPROGRAM=='PHASER':

         print ""
         print "--------------------------------"
         print "Setting up ensemble for Phaser:"
         print "--------------------------------"
         print ""

         # Make sure we are not asking for more models than are in the MR list
         if mstat.num_MR_models < init.keywords.ENSEMNUM:
            init.keywords.ENSEMNUM=mstat.num_MR_models
            sys.stdout.write("Resetting the number in the ensmemble to " + `init.keywords.ENSEMNUM`
                             + " as there are not enough models for the specified value\n")
            sys.stdout.write("\n")

         ens=Ensemble.Ensemble()
         ens.setup_ensemble(mstat, init.keywords.ENSEMNUM)

         # write the ensemble details to the html page
         if init.keywords.HTMLOUT == True:
            mstat.search_results.write_ensemble_details(init, mstat, target_info, mstat.results_html)
   
      # If we are not using Phaser and there is an ensemble in the list, remove it
      else:
         if "ensm__" in mstat.model_list.keys():
            del mstat.model_list["ensm__"]
 

      # Loop over all of the search models and submit them to one of the MR programs
      for model in mstat.sorted_model_list:

         MODEL=mstat.model_list[model]
   
         # Set the job number
         MODEL.setMR_JobNumber(mstat.mr_counter)
         
         # Print some details about this job
         self.print_job_details(MODEL, mstat)

         # if we are using Phaser as the MR program then we start the MR job accordingly.
         if mstat.MRPROGRAM=='PHASER':

            # Setup for Phaser
            MODEL.setPhaserLogFile(MODEL.model_directory + '/mr/phaser_' + MODEL.name + '.log')
            MODEL.setPhaserSummary(MODEL.model_directory + '/mr/phaser_' + MODEL.name + '.sum')
            MODEL.setPhaserSolnFile(MODEL.model_directory + '/mr/phaser_' + MODEL.name + '.sol')
            MODEL.setPhaserMTZfile(MODEL.model_directory + '/mr/phaser_' + MODEL.name + '.1.mtz')
            MODEL.setPhaserPDBfile(MODEL.model_directory + '/mr/phaser_' + MODEL.name + '.1.pdb')
            MODEL.setPhaserJobName("MRB_" + MODEL.name)

            # Create specific files for the Refmac MTZ, PDB, solution file and logfile for Phaser.
            MODEL.setRefmacLogFile(MODEL.model_directory +'/refine/refmac_phaser_' + MODEL.name + '.log')
            MODEL.setRefmacPDBfile(MODEL.model_directory + "/refine/refmac_phaser_" + MODEL.name + ".pdb")
            MODEL.setRefmacMTZINfile(target_info.hklin)
            MODEL.setRefmacMTZOUTfile(MODEL.model_directory + "/refine/refmac_phaser_" + MODEL.name + ".mtz")

            # Complete the jobs script to be Phaser specific
            phaser_job=MRBUMP_Phaser.Phaser()
            phaser_job.submit_job(init, mstat, MODEL, target_info)
      
         # or if we are using molrep...
         if mstat.MRPROGRAM=='MOLREP':

            # Make sure not to try the phaser ensemble model if we are running Molrep
            if "ensm__" in MODEL.name: 
               if self.debug:
                  sys.stdout.write("Ignoring ensmble model in Molrep\n") 
                  sys.stdout.write("\n") 

            else:
               # Setup for Molrep   
               MODEL.setMolrepLogFile(MODEL.model_directory + '/mr/molrep.log')
               MODEL.setMolrepSolnFile(MODEL.model_directory + '/mr/molrep.log')
               MODEL.setMolrepMTZfile(MODEL.model_directory + '/mr/molrep_' + MODEL.name + '.1.mtz')
               MODEL.setMolrepPDBfile(MODEL.model_directory + '/mr/molrep_' + MODEL.name + '.1.pdb')
               MODEL.setMolrepJobName("MRB_" + MODEL.name)

               # Create specific files for the Refmac solution file and logfile for Molrep.
               MODEL.setRefmacLogFile(MODEL.model_directory +'/refine/refmac_molrep' + MODEL.name + '.log')
               MODEL.setRefmacPDBfile(MODEL.model_directory + "/refine/refmac_molrep_" + MODEL.name + ".pdb")
               MODEL.setRefmacMTZINfile(target_info.hklin)
               MODEL.setRefmacMTZOUTfile(MODEL.model_directory + "/refine/refmac_molrep_" + MODEL.name + ".mtz")

               # Complete the jobs script to be Phaser specific
               molrep_job=MRBUMP_Molrep.Molrep()
               molrep_job.submit_job(init, mstat, MODEL, target_info)
      
         # If we are using the cluster we do the following:
         if init.keywords.CLUSTER == False:

            # If we are running on a single machine we do the following. Uses the same script
            sys.stdout.write("Finished job, checking results....\n")

            # Check the results for this job
            queue=Model_Queue.Queue_info()
            queue.check_result_serial(mstat, init, model)

            # If a solution is found output the results and exit                
            if mstat.solution_found == True:
               if init.keywords.TRYALL == False:
                  mstat.mr_results.copy_solution_files(mstat, mstat.results_dir + '/solution')
                  mstat.mr_results.display_results(mstat, init, target_info)
                  break
               else:
                  mstat.mr_results.display_results(mstat, init, target_info)

         # Increment the counter 
	 mstat.mr_counter=mstat.mr_counter+1
         mstat.model_list[model]=MODEL

      job_id_log.close()

   def print_job_details(self, MODEL, mstat):
      """ A function to output the details of the MR job to be submitted for execution. """

      # Print some details about this job
      sys.stdout.write("\n")
      sys.stdout.write("*******************************************************************************************\n")
      sys.stdout.write("\n")
      sys.stdout.write("#####################################################\n")
      sys.stdout.write("Template Model %s: %s\n" % (MODEL.MR_job_number, MODEL.name))
      sys.stdout.write("#####################################################\n")
      sys.stdout.write("\n")
      sys.stdout.write("Working directory:\n    %s\n" % MODEL.model_directory)

      # List the models being used as input to the MR job
      if MODEL.number_ensem > 1:
         sys.stdout.write("Input search models: \n")
         for i in range(MODEL.number_ensem):
            sys.stdout.write("   " + MODEL.PDBfile[i] + "\n") 
      else:
         sys.stdout.write("Input search model:\n   %s\n" % MODEL.PDBfile[0])

      sys.stdout.write("Search model type:            %s\n" % MODEL.type)
      sys.stdout.write("Sequence identity:            %.3f\n" % MODEL.seqID[0])
      sys.stdout.write("Molecular Replcement Program: %s\n" % mstat.MRPROGRAM)
      sys.stdout.write("Refinement Program:           Refmac5\n")
      sys.stdout.write("\n")
 
   def sort_model_list(self, mstat):
      """ Sort the models in the list according to sequence identity with the target structure. """

      for name in mstat.sorted_MR_list:
         for model in mstat.model_list.keys():
            if mstat.model_list[model].chain_source == name:
              mstat.sorted_model_list.append(model)
